


%==========================================================================
% This program checks the relations between the adjoints of the 
% different rocketcar versions.
% Prior to running it, please be sure to run the corresponding AMPL
% programs rocketcar_ode, rocketcar_pde and rocketcar_ode_pde with
% decent and matching discretizations to create the necessary data input!
%==========================================================================



clear all
close all



format long e



% data input
%===============

fid = fopen('q.txt');
a1 = textscan(fid, '%*f %*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
b1=a1{1};

fid = fopen('n1.txt');
a2 = textscan(fid, '%f ','delimiter',',','headerlines', 1);
fclose(fid);
n1=a2{1};

fid = fopen('n2.txt');
a3 = textscan(fid, '%f ','delimiter',',','headerlines', 1);
fclose(fid);
n2=a3{1}-1;

fid = fopen('p1.txt');
a4 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
p1=-a4{1};

fid = fopen('p2.txt');
a5 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
p2=-a5{1};

fid = fopen('w2.txt');
a6 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
w2=a6{1};

fid = fopen('p3.txt');
a7 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
p3=-a7{1};

fid = fopen('tf.txt');
a8 = textscan(fid, '%f ','delimiter',',','headerlines', 1);
fclose(fid);
tf=a8{1};

fid = fopen('nu1.txt');
a9 = textscan(fid, '%f ','delimiter',',','headerlines', 1);
fclose(fid);
nu1=a9{1};

fid = fopen('nu2.txt');
a10 = textscan(fid, '%f ','delimiter',',','headerlines', 1);
fclose(fid);
nu2=a10{1}; 



for i=1:n1
    for j=1:n2
        q(i,j)=-b1((j-1)*n1+i);
    end
end 


N=60;

for n=1:N
options=optimset('Display','off','TolFun',0.00000001);
k(n) = fsolve(@myfun,(n-1)*pi,options);
end
k(1)=1.306542374;


%Problem 1:
alpha = 1./(0.5.*k.^2+1.5).*(sin(k)+1./k.*(1-cos(k))).*(k.*cos(0.5.*k)+sin(0.5.*k));

%Problem 2:
beta  = (1+k.^2).^2./(2*k.^2+6).*sin(k).^2;



%==========================================================================




% projection formula   eq:projPDE
%=====================================



% Double integral
%===============================


for t=1:n1
    int(t)=0;
    for s=t:n1       % Integral with respect to t  
     F(s)=0;    
     for j=1:n2-1      % Integral over q with respect to x  
         if (j==1 || j==n2-1)
            F(s)=F(s)+0.5*q(s,j);
         else
            F(s)=F(s)+q(s,j); 
         end
     end
     F(s)=F(s)/n2;
     if (s==t || s==n1)
     int(t)=int(t)+0.5*2*w2(s)*F(s);
     else
     int(t)=int(t)+2*w2(s)*F(s);
     end
    end
    int(t)=int(t)/n1*tf;  
end    



% Calculation of the skaling factor gamma
%==================================================


x1= 1000;  % estimated range of gamma (times 10000)
x2= 10000;

for gamma=(x1+1):x2
    sum(gamma-x1)=0;
    for j=1:n1
        if (j==1 || j==n1)
        sum(gamma-x1)=sum(gamma-x1)+0.5*abs(-p2(j)+p2(n1-2)+p1(j)*(tf-j/n1*tf)+gamma*0.001*int(j));
        else
        sum(gamma-x1)=sum(gamma-x1)+abs(-p2(j)+p2(n1-2)+p1(j)*(tf-j/n1*tf)+gamma*0.001*int(j)); 
        end
    end
    sum(gamma-x1)=sum(gamma-x1)/n1*tf;
end

figure
plot(linspace((x1+1)/1000,x2/1000,x2-x1),sum);
title('Calculation of the skaling factor gamma','FontSize',15)


gammaopt=min(find(sum ==min([sum])))/1000+x1/1000;




% Residuum of eq:rel b  (4.14 b)
%=====================



figure
plot(linspace(0,tf,n1),p2);
hold on
plot(linspace(0,tf,n1),p2(n1-2)+p1'.*((tf-linspace(0,tf,n1)))+gammaopt*int,'--r');
%plot(linspace(0,tf,n1),nu1+nu2*(linspace(0,tf,n1))+gammaopt*int,'--g');
hold off
title ('right and lefthand side of formula (4.14 b) with opt. \gamma ','FontSize',15)




 
%==========================================================================
q=q*gammaopt;
%==========================================================================





% Plot of mod. q
%================

 figure
 [Ort,Zeit] = meshgrid(linspace(0,1,n2/1),linspace(0,tf,n1/1));
 surf(Ort,Zeit,q(1:1:n1,1:1:n2))
 colormap jet(1024)
 %shading interp
 grid on
 title('q (modified)');
 
 
 
 
 
 
 % Integral over q with respect to x 
 %==================================
 
 
 for i=1:n1
     F(i)=0;
     for j=1:n2-1
         if (j==1 || j==n2-1)
            F(i)=F(i)+0.5*q(i,j);
         else
            F(i)=F(i)+q(i,j); 
         end
     end
     F(i)=F(i)/n2;
 end    
   
figure 
plot(linspace(0,tf,n1),F)
title('$\int\limits_0^lq\,dx$','interpreter','latex','FontSize',15);




% Calculation of q_x
%===================


for i=1:n1
    for j=1:(n2-1)
        q_x(i,j)=(q(i,j+1)-q(i,j))*n2;
    end
        q_x(i,n2)=q_x(i,n2-1);
end




% Integral over q_x with respect to x 
%====================================


for j=1:n2
     F2(j)=0;
     for i=1:n1
         if (i==1 || i==n1)
            F2(j)=F2(j)+0.5*q_x(i,j);
         else
            F2(j)=F2(j)+q_x(i,j); 
         end
     end
     F2(j)=F2(j)/n1;
end    
     
figure 
plot(linspace(0,1,n2),F2)
title('$\int\limits_0^{t_f}q_x\,dt$','interpreter','latex','FontSize',15);







%==========================================================================



% the following only works for Problem 1!
%========================================




% check of eq:rel d (4.14 d)
%============================


% Integral
%=========

for t=1:n1
   if t==n1
      int2(t)=0;
   else   
     sum1=0;  % sum 
     for n=1:N
        sum2=0; % Integral
        for s=t:n1
            if (s==t || s==n1)
             sum2=sum2+0.5*alpha(n)*p3(s)*exp((-(k(n))^2)*(s-t)/n1*tf);
            else
             sum2=sum2+alpha(n)*p3(s)*exp((-(k(n))^2)*(s-t)/n1*tf);
            end 
        end 
        sum1=sum1+sum2/n1*tf;
     end 
     int(t)=sum1;
   end
end



% Derivative
%============

for i=1:size(int,2)-1
dint(i)=(int(i+1)-int(i))*size(int,2)/tf;
end
dint(size(int,2))=dint(size(int,2)-1);


% Residuum
%=========


figure
plot(linspace(0,tf,n1),-dint);
hold on
plot(linspace(0,tf,n1),F,'--r');
hold off
title ('right and lefthand side of formula (4.18 d) with opt. \gamma ','FontSize',15)










