
% =========================================================================
%  Plotting program for the ODE version
%  Prior to running it, please be sure to run the corresponding AMPL
%  program rocketcar_ode with the desired discretization
%  to create the necessary data input!
% =========================================================================



clear all
close all



format long e



% data input
% ===========

fid = fopen('p1.txt');
a1 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
b1=a1{1};

fid = fopen('p2.txt');
a2 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
b2=a2{1};

fid = fopen('p3.txt');
a3 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
b3=a3{1};

fid = fopen('w1.txt');
a4 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
b4=a4{1};

fid = fopen('w2.txt');
a5 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
b5=a5{1}; 

fid = fopen('w3.txt');
a6 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
b6=a6{1};

fid = fopen('u.txt');
a7 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
b7=a7{1};

fid = fopen('tf.txt');
a8 = textscan(fid, '%f ','delimiter',',','headerlines', 1);
fclose(fid);
tf=a8{1};



discr=size(b1,1);



for i=1:discr
   p1(i)=-b1(i); 
   p2(i)=-b2(i);
   p3(i)=-b3(i);
end

for i=1:discr 
   u(i)=b7(i); 
end  
u(discr+1)=u(discr);

for i=1:(discr+1)  
   w1(i)=b4(i);
   w2(i)=b5(i);
   w3(i)=b6(i);
end

for i=1:discr
    Hamilton(i)=1+0.05*u(i)^2+w2(i)*p1(i)+u(i)*p2(i)+(w3(i+1)-w3(i))*size(u,2)/tf*p3(i);
end


N=30;

for n=1:N
options=optimset('Display','off','TolFun',0.00000001);
k(n) = fsolve(@myfun,(n-1)*pi,options);
end
k(1)=1.306542374;



% Problem 1:
alpha = 1./(0.5.*k.^2+1.5).*(sin(k)+1./k.*(1-cos(k))).*(k.*cos(0.5.*k)+sin(0.5.*k));

% Problem 2:
% beta  = (1+k.^2).^2./(2*k.^2+6).*sin(k).^2;






% Dual variables
% ======================


figure
plot(linspace(0,tf,size(p1,2)),p1);
title('p1')

figure
plot(linspace(0,tf,size(p2,2)),p2);
title('p2')

figure
plot(linspace(0,tf,size(p2,2)),p3);
title('p3')



% Primal variables
% ======================


figure
plot(linspace(0,tf,size(w1,2)),w1);
title('w1')

figure
plot(linspace(0,tf,size(w2,2)),w2);
title('w2')

figure
plot(linspace(0,tf,size(w3,2)),w3);
title('w3')



% Control
% ======================


figure
plot(linspace(0,tf,size(u,2)),u);
title('u')


% Hamiltonian
% ======================


figure
plot(linspace(0,tf,discr),Hamilton);
title('Hamiltonian')





% Temperature
% ======================

% Problem 1:

for t=1:size(w2,2)
    T(t)=0; 
    for s=1:t
        if s==1
           T(t)=T(t)+0.5*sum(w2(s)^2.*alpha.*exp(-k.^2*(t-s)/(size(w2,2)-1)*tf));
        elseif s==t
           T(t)=T(t)+0.5*sum(w2(s)^2.*alpha.*exp(-k.^2*(t-s)/(size(w2,2)-1)*tf));
        else
           T(t)=T(t)+sum(w2(s)^2.*alpha.*exp(-k.^2*(t-s)/(size(w2,2)-1)*tf));
        end   
    end
    T(t)=T(t)/(size(w2,2)-1)*tf;
end


% Problem 2:

% T(1)=0;
% for t=2:size(w2,2)
%     T(t)=0; 
%     for s=1:t
%         if s==1
%            T(t)=T(t)+0.5*sum(beta./k.^2.*2.*w2(s).*u(s).*exp(-k.^2*(t-s)/(size(w2,2)-1)*tf));
%         elseif s==t
%            T(t)=T(t)+0.5*sum(beta./k.^2.*2.*w2(s).*u(s).*exp(-k.^2*(t-s)/(size(w2,2)-1)*tf));
%         else
%            T(t)=T(t)+sum(beta./k.^2.*2.*w2(s).*u(s).*exp(-k.^2*(t-s)/(size(w2,2)-1)*tf));
%         end   
%     end
%     T(t)=sum(beta.*(w2(t).^2./k.^2-w2(1)^2./k.^2.*exp(-k.^2*t/(size(w2,2)-1)*tf)))-T(t)/(size(w2,2)-1)*tf;
%  end 



 figure
 plot(linspace(0,tf,size(w1,2)),T);
 title('T')



for i=1:size(p1,2)-1
p2dot(i)=(p2(i+1)-p2(i))*size(p1,2)/tf;
end
p2dot(size(p1,2))=p2dot(size(p1,2)-1);

figure
plot(linspace(0,tf,size(p2dot,2)),p2dot);
title('p2 dot')



% the following only works for Problem 1!
%========================================


for t=1:size(p2,2)
   if t==size(p2,2)
      int(t)=0;
      int2(t)=0;
   else   
   sum1=0;  % Sum 
   sum3=0;  % 2nd Integral
   for n=1:N
      sum2=0; % Integral
      sum4=0;
      for s=t:size(p2,2)
          if s==t
           sum2=sum2+0.5*2*w2(t)*alpha(n)*p3(s)*exp((-(k(n))^2)*(s-t)/(size(w2,2)-1)*tf);
           sum4=sum4+0.5*2*u(t)*alpha(n)*p3(s)*exp((-(k(n))^2)*(s-t)/(size(w2,2)-1)*tf);
          elseif s==size(p2,2)
           sum2=sum2+0.5*2*w2(t)*alpha(n)*p3(s)*exp((-(k(n))^2)*(s-t)/(size(w2,2)-1)*tf);
           sum4=sum4+0.5*2*u(t)*alpha(n)*p3(s)*exp((-(k(n))^2)*(s-t)/(size(w2,2)-1)*tf);
          else
           sum2=sum2+2*w2(t)*alpha(n)*p3(s)*exp((-(k(n))^2)*(s-t)/(size(w2,2)-1)*tf);
           sum4=sum4+2*u(t)*alpha(n)*p3(s)*exp((-(k(n))^2)*(s-t)/(size(w2,2)-1)*tf);
          end 
      end 
      sum1=sum1+sum2/(size(w2,2)-1)*tf;
      sum3=sum3+sum4/(size(w2,2)-1)*tf;
   end  
   int(t)=sum1;
   int2(t)=sum3;
   end
end 
 
for i=1:size(int,2)-1
dint(i)=(int(i+1)-int(i))*size(int,2)/tf;
mu(i)=-(p3(i+1)-p3(i))*size(int,2)/tf;
end
dint(size(int,2))=dint(size(int,2)-1);
mu(size(int,2))=mu(size(int,2)-1);





% Measure mu
% ======================


figure
plot(linspace(0,tf,size(p2,2)),mu);
title('\mu')





