
% =========================================================================
%  Plotting program for the ODE-PDE version
%  Prior to running it, please be sure to run the corresponding AMPL
%  program rocketcar_ode_pde with the desired discretization
%  to create the necessary data input!
% =========================================================================


clear all
close all



format long e


% data input
% ===========

fid = fopen('T.txt');
a1 = textscan(fid, '%*f %*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
b1=a1{1};

fid = fopen('n1.txt');
a2 = textscan(fid, '%f ','delimiter',',','headerlines', 1);
fclose(fid);
n1=a2{1};

fid = fopen('n2.txt');
a3 = textscan(fid, '%f ','delimiter',',','headerlines', 1);
fclose(fid);
n2=a3{1};

fid = fopen('w1.txt');
a4 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
b4=a4{1};

fid = fopen('w2.txt');
a5 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
b5=a5{1}; 

fid = fopen('u.txt');
a7 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
b7=a7{1};

fid = fopen('tf.txt');
a8 = textscan(fid, '%f ','delimiter',',','headerlines', 1);
fclose(fid);
tf=a8{1};

fid = fopen('q.txt');
a9 = textscan(fid, '%*f %*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
b9=a9{1};

fid = fopen('p1.txt');
a10 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
b10=a10{1};

fid = fopen('p2.txt');
a11 = textscan(fid, '%*f %f ','delimiter',',','headerlines', 1);
fclose(fid);
b11=a11{1};





z=n1+1;      % discretization in time
o=n2+1;      % discretization in space




for i=1:z
    for j=1:o
        T(i,j)=b1((j-1)*z+i);
       
    end 
    w1(i)=b4(i);
    w2(i)=b5(i);
end
  

for i=1:z-1
    p1(i)=b10(i);
    p2(i)=b11(i);
end    

for i=1:(z-1)
    for j=1:(o-2)
        q(i,j)=-2*b9((j-1)*(z-1)+i);
    end
end
for i=1:z-2
    u(i)=b7(i+1);
end    




% Temperature and its adjoint
% ===========================


 figure
 [Ort,Zeit] = meshgrid(linspace(0,1,o),linspace(0,tf,z));
 surf(Ort,Zeit,T)
 colormap jet(1024)
 title('T')
 
 figure
 [Ort,Zeit] = meshgrid(linspace(0,1,o-2),linspace(0,tf,z-1));
 surf(Ort,Zeit,q)
 colormap jet(1024)
 title('q')


% T, cross section at given point of time
% =======================================


 figure
 t=4;
 for m=1:o 
 tem(m,1)=T(int16(t/tf*z),m); 
 tem(m,2)=4/25*(-((m-1)/(o-1))^2+(m-1)/(o-1)+1);
 end
 
 plot(linspace(0,1,o),(tem(:,1)-tem(:,2)))
 title('T, cross section at t=4')
 

% T, cross section at given x
% ===========================


 figure
 for m=1:z
 tem2(m)=T(m,0.5*n2+1);
 line1(m)=1.5;
 end
 
 plot(linspace(0,tf,z),tem2);
 hold on
 plot(linspace(0,tf,z),line1,':')
 hold off
 title('T, cross section at x=0.5')

 
% Phase-diagram
% =============


 figure 
 plot(w1,w2);
 title('Phase diagram')


% Position and velocity
% ======================


 figure
 plot(linspace(0,tf,size(w1,2)),w1)
 title('w1')

 figure
 plot(linspace(0,tf,size(w2,2)),w2)
 title('w2')



% ODE adjoints 
% ============


 figure
 plot(linspace(0,tf,size(p1,2)),p1)
 title('p1')

 figure
 plot(linspace(0,tf,size(p2,2)),p2)
 title('p2')



% Control
% ========


for m=1:size(u,2)          % control constraints as additional lines
    line2(m)=-1;
    line3(m)=1;
    line4(m)=0;
end
 figure
 plot(linspace(0,tf,size(u,2)),u)
 hold on
 plot(linspace(0,tf,size(u,2)),line2,':')
 plot(linspace(0,tf,size(u,2)),line3,':')
 plot(linspace(0,tf,size(u,2)),line4,':')
 hold off
 title('u')

























 